(*********************************************************)
(* TSLXPolarChart is a polar chart component for Delphi6 *)
(* distributed for free, non-commercial purpose          *)
(*  2007 by Sl-Alex http://www.sl-alex.nm.ru/           *)
(*********************************************************)
unit PolarChart;

interface

uses Windows, Messages,Classes, ExtCtrls, Graphics, SysUtils,
     Types, Controls, PolarSeries, PolarFill, PolarAxis, MathExt, GraphicsExt, PolarMargin, PolarTitle;

type

TSLXPolarChart = class(TCustomPanel) // 
  private
    FSeriesList: TList; //  
    FVertAxis: TSLXPolarAxis; //  
    FHorzAxis: TSLXPolarAxis; //  
    FExtCircle: TSLXPolarExtCircle; //  
    FIntCircle: TSLXPolarIntCircle; //  
    FCircleGrid: TSLXPolarGrid; //   
    FRadialGrid: TSLXPolarGrid; //   
    FAutoMinMax: Boolean; //  
    FMinimum: Double; //  
    FMaximum: Double; //  
    FAutoMin: Double; //    
    FAutoMax: Double; //    
    FManualMin: Double; //      
    FManualMax: Double; //      
    FChartBitMap: TBitMap; // ,        
    FBackGround: TSLXFill; //  
    FOnPaint: TNotifyEvent; //  
    FMargin: TSLXMargin; //   
    FTitle: TSLXTitle; //  
    FLabelsFont: TFont; //  
    FUpdating: Boolean; //     (BeginUpdate - EndUpdate)
    procedure FLabelsFontChange(Sender: TObject); //   
    procedure FSetBackGround(NewBackGround: TSLXFill); //  BackGround;
    function FGetSeriesCount: Integer; //   
    procedure FInitValues; //  
    procedure FSetAutoMinMax(NewValue: Boolean); // 
    procedure FSetManualMin(NewValue: Double); //   - 
    procedure FSetManualMax(NewValue: Double); //   - 
    function  FGetSeries(const AIndex: Integer): TSLXPolarSeries; //  
  public
    constructor Create(AOwner: TComponent); override; //  
    destructor Destroy; override; //  
    procedure Paint; override; //   
    procedure Clear; //  
    procedure BeginUpdate; //   
    procedure EndUpdate;   //   
    function AddSeries: Integer; //  
    procedure RepaintMem(AlsoScreen: Boolean); //    
    property Count: Integer read FGetSeriesCount; //  
    property Canvas; //   ,     OnPaint
    property BitMap: TBitMap read FChartBitMap write FChartBitMap; // ,    
    property Series[const AIndex: Integer]: TSLXPolarSeries read FGetSeries;// read GetSeries;
  published
    property Align;
    property Alignment;
    property Anchors;
    property Constraints;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnStartDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property AutoSize: Boolean read FAutoMinMax write FSetAutoMinMax; // 
    property VertAxis: TSLXPolarAxis read FVertAxis write FVertAxis;//  
    property HorzAxis: TSLXPolarAxis read FHorzAxis write FHorzAxis;//  
    property GridCircle: TSLXPolarGrid read FCircleGrid write FCircleGrid; //  
    property GridRadial: TSLXPolarGrid read FRadialGrid write FRadialGrid; //  
    property Margins: TSLXMargin read FMargin write FMargin; //    
    property ExtCircle: TSLXPolarExtCircle read FExtCircle write FExtCircle;//  
    property IntCircle: TSLXPolarIntCircle read FIntCircle write FIntCircle;//  
    property ManualMin: Double read FManualMin write FSetManualMin; //  
    property ManualMax: Double read FManualMax write FSetManualMax; //  
    property BackGround: TSLXFill read FBackGround write FSetBackGround; //  
    property OnPaint: TNotifyEvent read FOnPaint write FOnPaint; //   
    property Title: TSLXTitle read FTitle write FTitle; //  
    property LabelsFont: TFont read FLabelsFont write FLabelsFont; //  
end;

procedure Register;

implementation

{$R PolarChart.dcr} //  

//   
procedure TSLXPolarChart.BeginUpdate;
begin
  FUpdating:=True;
end;

procedure TSLXPolarChart.EndUpdate;   //   
begin
  FUpdating:=False;
  RepaintMem(True);
end;

//   
procedure TSLXPolarChart.FSetBackGround(NewBackGround: TSLXFill);
begin
  FBackGround:=NewBackGround;
  RepaintMem(True);
end;

//  
procedure TSLXPolarChart.FSetAutoMinMax(NewValue: Boolean);
begin
  FAutoMinMax:=NewValue;
  RepaintMem(True);
end;

//     
procedure TSLXPolarChart.FSetManualMin(NewValue: Double);
begin
  FManualMin:=NewValue;
  RepaintMem(True);
end;

//     
procedure TSLXPolarChart.FSetManualMax(NewValue: Double);
begin
  FManualMax:=NewValue;
  RepaintMem(True);
end;

//  
constructor TSLXPolarChart.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  // 
  FUpdating:=True;
  ControlStyle := ControlStyle + [csReplicatable];
  //  
  FVertAxis:=TSLXPolarAxis.Create(Self);
  //  
  FHorzAxis:=TSLXPolarAxis.Create(Self);
  //  
  FExtCircle:=TSLXPolarExtCircle.Create(Self);
  //  
  FIntCircle:=TSLXPolarIntCircle.Create(Self);
  // 
  FBackGround:=TSLXFill.Create(Self);
  // ,    
  FChartBitMap:=TBitMap.Create;
  //  
  FCircleGrid:=TSLXPolarGrid.Create(Self); //   
  //  
  FRadialGrid:=TSLXPolarGrid.Create(Self); //   
  //    
  FMargin:=TSLXMargin.Create(Self);
  //  
  FTitle:=TSLXTitle.Create(Self);
  //     
  FLabelsFont:=TFont.Create;
  //  
  FSeriesList:=TList.Create;
  //  
  FInitValues;
end;

//  
procedure TSLXPolarChart.FInitValues;
var i: Integer;
begin
  //  
  Width:=400;
  Height:=400;
  //    
  FAutoMinMax:=True;
  FManualMin:=0;
  FManualMax:=10;
  //     
  FLabelsFont.OnChange:=FLabelsFontChange;
  FLabelsFont.Color:=clNavy;
  //  
  FCircleGrid.Color:=clTeal;
  FCircleGrid.Style:=psDot;
  //  
  FRadialGrid.Color:=clTeal;
  FRadialGrid.Style:=psDot;
  // ,    
  FChartBitMap.Width:=Width;
  FChartBitMap.Height:=Height;
  //  
  FVertAxis.Style:=psSolid;
  FVertAxis.Width:=1;
  FVertAxis.Color:=clBlack;
  //  
  FHorzAxis.Style:=psSolid;
  FHorzAxis.Width:=1;
  FHorzAxis.Color:=clBlack;
  //  
  FExtCircle.Style:=psSolid;
  FExtCircle.Width:=1;
  FExtCircle.Color:=clGreen;
  //  
  FIntCircle.Raduis:=10; //    
  FIntCircle.Style:=psSolid;
  FIntCircle.Width:=1;
  FIntCircle.Color:=clGreen;
  //    
  DoubleBuffered:=True;
  //          (      )
  if csDesigning in ComponentState then begin
    AddSeries;
    for i:=0 to 35 do begin
      Series[0].AddPoint(i*2*PI/36, random(10));
    end;
    Series[0].CloseSeries:=True;
  end;
  //   
  FUpdating:=False;
  RepaintMem(False);
end;

//  
destructor TSLXPolarChart.Destroy;
begin
  //    
  Clear;
  //   
  FSeriesList.Destroy;
  //  
  FLabelsFont.Destroy;
  //  
  FTitle.Destroy;
  //  
  FMargin.Destroy;
  //  
  FBackGround.Destroy;
  //    
  FChartBitMap.Destroy;
  //  
  FCircleGrid.Destroy;
  FRadialGrid.Destroy;
  //  
  FIntCircle.Destroy;
  FExtCircle.Destroy;
  FVertAxis.Destroy;
  FHorzAxis.Destroy;
  //    :)
  inherited Destroy;
end;

//   
procedure TSLXPolarChart.FLabelsFontChange(Sender: TObject);
begin
  RepaintMem(True);
end;

//  
procedure TSLXPolarChart.RepaintMem(AlsoScreen: Boolean);
var i,j,tempX,tempY: Integer;
  X1,X2,Y1,Y2: Integer;
  tempangle,tempvalue: Double;
  minmaxdiff: Extended;
  mant: Extended;
  expt: Integer;
  circlegridstepX, circlegridstepY: Double;
begin
  if FUpdating then Exit;
  //        
  if FGetSeriesCount()>0 then begin
    if Series[0].Count>0 then begin //  
      FMinimum:=Series[0].Minimum;
      FMaximum:=Series[0].Maximum;
      if FMaximum=FMinimum then FMaximum:=FMinimum*2;
    end;
    for i:=1 to FGetSeriesCount()-1 do begin //    
      if Series[i].Count>0 then begin
        if Series[i].Minimum<FMinimum then FMinimum:=Series[i].Minimum;
        if Series[i].Maximum>FMaximum then FMaximum:=Series[i].Maximum;
      end;
    end;
  end;
  //     
  minmaxdiff:=FMaximum-FMinimum;
  //    
  FrExp10(minmaxdiff,mant,expt);
  if FAutoMinMax then begin
    FAutoMin:=FloorTo(FMinimum,expt);
    FAutoMax:=CeilTo(FMaximum,expt);
  end else begin
    FAutoMin:=FManualMin;
    FAutoMax:=FManualMax;
  end;
  //   
  FChartBitMap.Width:=Width;
  FChartBitMap.Height:=Height;
  //    ( )
  FBackGround.FillCanvas;
  if (Width-Margins.Left-Margins.Right)<=0 then exit;
  if (Height-Margins.Top-Margins.Bottom)<=0 then exit;
  //    (  Margin)
  circlegridstepX:=((Width-Margins.Left-Margins.Right)-FIntCircle.Raduis*(Width-Margins.Left-Margins.Right)/100)/(2*10);
  circlegridstepY:=((Height-Margins.Top-Margins.Bottom)-FIntCircle.Raduis*(Height-Margins.Top-Margins.Bottom)/100)/(2*10);
  for i:=1 to 9 do begin
    DrawEllipseExt(FChartBitMap.Canvas,FCircleGrid,
                     Margins.Left+
                     (Width-Margins.Left-Margins.Right) div 2 -
                     Round(FIntCircle.Raduis*((Width-Margins.Left-Margins.Right) div 2)/100) -
                     Round(circlegridstepX*i),

                     Margins.Top+
                     (Height-Margins.Top-Margins.Bottom) div 2 -
                     Round(FIntCircle.Raduis*((Height-Margins.Top-Margins.Bottom) div 2)/100) -
                     Round(circlegridstepY*i),

                     Margins.Left+
                     (Width-Margins.Left-Margins.Right) div 2 +
                     Round(FIntCircle.Raduis*((Width-Margins.Left-Margins.Right) div 2)/100) +
                     Round(circlegridstepX*i),

                     Margins.Top+
                     (Height-Margins.Top-Margins.Bottom) div 2 +
                     Round(FIntCircle.Raduis*((Height-Margins.Top-Margins.Bottom) div 2)/100) +
                     Round(circlegridstepY*i));
  end;
  SetBkMode(FChartBitMap.Canvas.Handle, TRANSPARENT);
  CopyFontProps(FLabelsFont,FChartBitMap.Canvas.Font);
  for i:=0 to 10 do begin
    FChartBitMap.Canvas.TextOut(Margins.Left+(Width-Margins.Left-Margins.Right) div 2,
        Margins.Top+(Height-Margins.Top-Margins.Bottom) div 2 -
        Round(FIntCircle.Raduis*(Height div 2)/100) -
        Round(circlegridstepY*i)-(abs(FChartBitMap.Canvas.Font.Height) div 2),
        ' '+FloatToStr(FAutoMin+(FAutoMax-FAutoMin)*i/10));
  end;
  SetBkMode(FChartBitMap.Canvas.Handle, OPAQUE);
  //   
  for i:=0 to 35 do begin
    X1:=Margins.Left+(Width-Margins.Left-Margins.Right) div 2+Round(cos(2*PI*i/36)*FIntCircle.Raduis*((Width-Margins.Left-Margins.Right) div 2)/100);
    Y1:=Margins.Top+(Height-Margins.Top-Margins.Bottom) div 2-Round(sin(2*PI*i/36)*FIntCircle.Raduis*((Height-Margins.Top-Margins.Bottom) div 2)/100);
    X2:=Margins.Left+(Width-Margins.Left-Margins.Right) div 2+Round(cos(2*PI*i/36)*((Width-Margins.Left-Margins.Right) div 2));
    Y2:=Margins.Top+(Height-Margins.Top-Margins.Bottom) div 2-Round(sin(2*PI*i/36)*((Height-Margins.Top-Margins.Bottom) div 2));
    DrawLineExt(FChartBitMap.Canvas,FRadialGrid,X1,Y1,X2,Y2);
  end;
  //  
  DrawLineExt(FChartBitMap.Canvas,FVertAxis,Margins.Left+(Width-Margins.Left-Margins.Right) div 2,
                                            Margins.Top,
                                            Margins.Left+(Width-Margins.Left-Margins.Right) div 2,
                                            Margins.Top+(Height-Margins.Top-Margins.Bottom));
  DrawLineExt(FChartBitMap.Canvas,FHorzAxis,Margins.Left,
                                            Margins.Top+(Height-Margins.Top-Margins.Bottom) div 2,
                                            Margins.Left+(Width-Margins.Left-Margins.Right),
                                            Margins.Top+(Height-Margins.Top-Margins.Bottom) div 2);
  //  
  if FSeriesList.Count>0 then begin
    for i:=0 to FGetSeriesCount-1 do begin
      if Series[i].Count >0 then begin
        tempangle:=Series[i].Point[0].angle; //    
        tempvalue:=Series[i].Point[0].value; // 
        if FAutoMax=FAutoMin then begin
          if FAutoMax=0 then FAutoMax:=FAutoMin+1 else FAutoMax:=FAutoMin*2;
        end;
        tempvalue:=(tempvalue-FAutoMin)/(FAutoMax-FAutoMin); // =0  FAutoMin; =1  FAutoMax
        tempvalue:=tempvalue*(1-FIntCircle.Raduis/100)+FIntCircle.Raduis/100; //  Radius  
        tempX:=Margins.Left+(Width-Margins.Left-Margins.Right) div 2+Round(cos(tempangle)*((tempvalue*((Width-Margins.Left-Margins.Right) div 2))));
        tempY:=Margins.Top+(Height-Margins.Top-Margins.Bottom) div 2-Round(sin(tempangle)*((tempvalue*((Height-Margins.Top-Margins.Bottom) div 2))));
        CopyPenProps(Series[i],FChartBitMap.Canvas.Pen);
        FChartBitMap.Canvas.Brush.Color:=BackGround.Color;
        FChartBitMap.Canvas.MoveTo(tempX,tempY);
        for j:=0 to Series[i].Count-1 do begin
          tempangle:=Series[i].Point[j].angle; //    
          tempvalue:=Series[i].Point[j].value; // 
          tempvalue:=(tempvalue-FAutoMin)/(FAutoMax-FAutoMin); // =0  FAutoMin; =1  FAutoMax
          tempvalue:=tempvalue*(1-FIntCircle.Raduis/100)+FIntCircle.Raduis/100; //  Radius  
          tempX:=Margins.Left+(Width-Margins.Left-Margins.Right) div 2+Round(cos(tempangle)*((tempvalue*((Width-Margins.Left-Margins.Right) div 2))));
          tempY:=Margins.Top+(Height-Margins.Top-Margins.Bottom) div 2-Round(sin(tempangle)*((tempvalue*((Height-Margins.Top-Margins.Bottom) div 2))));
          FChartBitMap.Canvas.LineTo(tempX,tempY);
        end;
        if Series[i].CloseSeries then begin //    
          tempangle:=Series[i].Point[0].angle; //    
          tempvalue:=Series[i].Point[0].value; // 
          tempvalue:=(tempvalue-FAutoMin)/(FAutoMax-FAutoMin); // =0  FAutoMin; =1  FAutoMax
          tempvalue:=tempvalue*(1-FIntCircle.Raduis/100)+FIntCircle.Raduis/100; //  Radius  
          tempX:=Margins.Left+(Width-Margins.Left-Margins.Right) div 2+Round(cos(tempangle)*((tempvalue*((Width-Margins.Left-Margins.Right) div 2))));
          tempY:=Margins.Top+(Height-Margins.Top-Margins.Bottom) div 2-Round(sin(tempangle)*((tempvalue*((Height-Margins.Top-Margins.Bottom) div 2))));
          FChartBitMap.Canvas.LineTo(tempX,tempY);
        end;
      end;
    end;
  end;
  //   
  DrawEllipseExt(FChartBitMap.Canvas,FExtCircle,Margins.Left,
                                                Margins.Top,
                                                Margins.Left+(Width-Margins.Left-Margins.Right),
                                                Margins.Top+(Height-Margins.Top-Margins.Bottom));
  //    (  )
  DrawEllipseExt(FChartBitMap.Canvas,FIntCircle,Margins.Left+(Width-Margins.Left-Margins.Right) div 2 - Round(FIntCircle.Raduis*((Width-Margins.Left-Margins.Right) div 2)/100),
                                                Margins.Top+(Height-Margins.Top-Margins.Bottom) div 2 - Round(FIntCircle.Raduis*((Height-Margins.Top-Margins.Bottom) div 2)/100),
                                                Margins.Left+(Width-Margins.Left-Margins.Right) div 2 + Round(FIntCircle.Raduis*((Width-Margins.Left-Margins.Right) div 2)/100),
                                                Margins.Top+(Height-Margins.Top-Margins.Bottom) div 2 + Round(FIntCircle.Raduis*((Height-Margins.Top-Margins.Bottom) div 2)/100));
  FTitle.TextOut(FChartBitMap.Canvas);
  if AlsoScreen then RePaint;
end;

procedure TSLXPolarChart.Paint;
begin
  bitblt(Canvas.Handle, 0, 0, width, height, FChartBitMap.Canvas.Handle, 0, 0, SRCCOPY);
  if Assigned(OnPaint) then OnPaint(Self);
end;

function TSLXPolarChart.AddSeries: Integer;
begin
  FSeriesList.Add(TSLXPolarSeries.Create(Self));
  AddSeries:=FSeriesList.Count;
end;

function TSLXPolarChart.FGetSeriesCount: Integer;
begin
  FGetSeriesCount:=FSeriesList.Count;
end;

function TSLXPolarChart.FGetSeries(const AIndex: Integer): TSLXPolarSeries;
begin
  if AIndex<FSeriesList.Count then
    FGetSeries:=FSeriesList.Items[AIndex]
  else
    FGetSeries:=nil;
end;

procedure TSLXPolarChart.Clear;
var i: Integer;
  curSeries: TSLXPolarSeries;
begin
  if FSeriesList.Count>0 then begin
    //    
    for i:=0 to FSeriesList.Count-1 do begin
      curSeries:=FSeriesList.Items[i];
      curSeries.Destroy;
    end;
    //   
    FSeriesList.Clear;
  end;
end;

procedure Register;
begin
  RegisterComponents('Sl-Alex',[TSLXPolarChart]);
end;

end.

