(*********************************************************)
(* Title class for TSLXPolarChart component              *)
(* distributed for free, non-commercial purpose          *)
(*  2007 by Sl-Alex http://www.sl-alex.nm.ru/           *)
(*********************************************************)
unit PolarTitle;

interface

uses Graphics, Classes, Windows, GraphicsExt;

type

//   -         
TSLXPosition=(tpLeftTop,tpTop,tpRightTop,tpLeft,tpRight,tpLeftBottom,tpBottom,tpRightBottom);

//  
TSLXTitle=class(TFont)
  private
    FPosition: TSLXPosition;
    FOwner: TComponent;
    FCaption: String;
    FVisible: Boolean;
    procedure FTitleChange(Sender: TObject);
    procedure FSetPosition(NewPosition: TSLXPosition);
    procedure FSetCaption(NewCaption: String);
    procedure FSetVisible(NewVisible: Boolean);
  public
    constructor Create(AOwner: TComponent);
    destructor Destroy; override;
    procedure TextOut(Canvas: TCanvas);
  published
    property Position: TSLXPosition read FPosition write FSetPosition;
    property Text: String read FCaption write FSetCaption;
    property Visible: Boolean Read FVisible write FSetVisible;
end;

implementation

uses PolarChart;

constructor TSLXTitle.Create(AOwner: TComponent);
begin
  inherited Create;
  FOwner:=AOwner;
  OnChange:=FTitleChange;
  Size:=10;
  Style:=[fsBold];
  Color:=clNavy;
  FCaption:='Chart';
  FPosition:=tpTop;
  FVisible:=True;
end;

destructor TSLXTitle.Destroy;
begin
  inherited Destroy;
end;

procedure TSLXTitle.FTitleChange(Sender: TObject);
begin
  (FOwner as TSLXPolarChart).RepaintMem(True);
end;

procedure TSLXTitle.FSetVisible(NewVisible: Boolean);
begin
  FVisible:=NewVisible;
  (FOwner as TSLXPolarChart).RepaintMem(True);
end;

procedure TSLXTitle.FSetPosition(NewPosition: TSLXPosition);
begin
  FPosition:=NewPosition;
  (FOwner as TSLXPolarChart).RepaintMem(True);
end;

procedure TSLXTitle.FSetCaption(NewCaption: String);
begin
  FCaption:=NewCaption;
  (FOwner as TSLXPolarChart).RepaintMem(True);
end;

procedure TSLXTitle.TextOut(Canvas: TCanvas);
begin
  if not FVisible then exit;
  SetBkMode((FOwner as TSLXPolarChart).BitMap.Canvas.Handle, TRANSPARENT);
  CopyFontProps(Self,(FOwner as TSLXPolarChart).BitMap.Canvas.Font);
  with (FOwner as TSLXPolarChart).BitMap do begin
    if FPosition=tpLeftTop then begin
      Canvas.TextOut(2,2,FCaption);
    end;
    if FPosition=tpTop then begin
      Canvas.TextOut((Width - Canvas.TextWidth(FCaption)) div 2,2,FCaption);
    end;
    if FPosition=tpRightTop then begin
      Canvas.TextOut(Width - Canvas.TextWidth(FCaption) - 4,2,FCaption);
    end;
    if FPosition=tpLeft then begin
      Canvas.TextOut(2,(Height - Canvas.TextHeight(FCaption)) div 2,FCaption);
    end;
    if FPosition=tpRight then begin
      Canvas.TextOut(Width - Canvas.TextWidth(FCaption) - 4,(Height - Canvas.TextHeight(FCaption)) div 2,FCaption);
    end;
    if FPosition=tpLeftBottom then begin
      Canvas.TextOut(2,(Height - Canvas.TextHeight(FCaption)) - 2,FCaption);
    end;
    if FPosition=tpBottom then begin
      Canvas.TextOut((Width - Canvas.TextWidth(FCaption)) div 2,(Height - Canvas.TextHeight(FCaption)) - 2,FCaption);
    end;
    if FPosition=tpRightBottom then begin
      Canvas.TextOut(Width - Canvas.TextWidth(FCaption) - 4,(Height - Canvas.TextHeight(FCaption)) - 2,FCaption);
    end;
  end;
  SetBkMode((FOwner as TSLXPolarChart).BitMap.Canvas.Handle, OPAQUE);
end;

end.
